package com.rfideas.android;

import android.app.Activity;
import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.pm.PackageManager;
import android.content.res.Configuration;
import android.hardware.usb.UsbDevice;
import android.hardware.usb.UsbManager;
import android.net.Uri;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.TextView;
import android.widget.Toast;
import com.rfideas.pcproxapisdk.Enumerate;
import com.rfideas.pcproxapisdk.HexConstants;
import com.rfideas.pcproxapisdk.SDKException;
import com.rfideas.pcproxapisdk.Reader;
import com.rfideas.pcproxapisdk.configuration.ConfigurationFlags;
import android.view.View.OnClickListener;

import java.io.FileInputStream;
import java.io.OutputStream;
import java.util.HashMap;
import java.util.Iterator;

import static android.Manifest.permission.WRITE_EXTERNAL_STORAGE;

public class Android extends AppCompatActivity implements OnClickListener{

    private Reader reader[];
    Button connect;
    Button disconnect;
    Button beepNow;
    Button getActiveId;
    TextView sdkOutput;
    TextView readerPartNumber;
    TextView maxConfigurationNumber;
    TextView AppInfoText;
    CheckBox disableKeystrokes;
    Button readHWGFile;
    Button writeHWGFile;
    Button readSecureHWGFile;
    Button writeSecureHWGFile;
    static final int WRITE_REQUEST_CODE = 1;
    static final int READ_REQUEST_CODE = 2;
    static final int SECURE_WRITE_REQUEST_CODE = 3;
    static final int SECURE_READ_REQUEST_CODE = 4;
    String appVersion = "1.2.0";

    private static final String TAG = "Android";

    int readerNo = 0; // To access the first connected reader.
    int configurationIndex = 0; //  To access the first configuration of the reader in use.

    private static final String ACTION_USB_PERMISSION = "com.android.example.USB_PERMISSION";

    private IntentFilter filter = new IntentFilter(ACTION_USB_PERMISSION);

    private PendingIntent mPermissionIntent;

    private boolean hasPermissionToCommunicate = false;

    private UsbManager mUsbManager;

    // Catches intent indicating if the user grants permission to use the USB device
    // It is used for Readers that are already plugged-in
    private final BroadcastReceiver mUsbReceiver = new BroadcastReceiver() {
        public void onReceive(Context context, Intent intent) {
            String action = intent.getAction();
            if (ACTION_USB_PERMISSION.equals(action)) {
                synchronized (this) {
                    UsbDevice device = intent.getParcelableExtra(UsbManager.EXTRA_DEVICE);
                    if (intent.getBooleanExtra(UsbManager.EXTRA_PERMISSION_GRANTED, false)) {
                        if (device != null) {
                            hasPermissionToCommunicate = true;
                        }
                    }
                }
            }
        }
    };

    @Override
    protected void onCreate(Bundle savedInstanceState){
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_android);
        Log.d(TAG, "Creating Activity");

        //defining all the architectural components from the UI
        connect = findViewById(R.id.connectButton);
        disconnect = findViewById(R.id.disconnectButton);
        beepNow = findViewById(R.id.beepnowButton);
        getActiveId = findViewById(R.id.getActiveIdButton);
        sdkOutput = findViewById(R.id.sdkOutputTextView);
        readerPartNumber = findViewById(R.id.partNumberText);
        disableKeystrokes = findViewById(R.id.disableKeystrokesCheckBox);
        maxConfigurationNumber = findViewById(R.id.displayMaxConfigurationNumber);
        readHWGFile = findViewById(R.id.readHWGFileButton);
        writeHWGFile = findViewById(R.id.writeHWGFileButton);
        readSecureHWGFile = findViewById(R.id.readSecureHWGFileButton);
        writeSecureHWGFile = findViewById(R.id.writeSecureHWGFileButton);
        AppInfoText = findViewById(R.id.AppInfoText);

        //activating functionality on clicking the component
        connect.setOnClickListener(this);
        beepNow.setOnClickListener(this);
        disconnect.setOnClickListener(this);
        getActiveId.setOnClickListener(this);
        disableKeystrokes.setOnClickListener(this);
        readHWGFile.setOnClickListener(this);
        writeHWGFile.setOnClickListener(this);
        readSecureHWGFile.setOnClickListener(this);
        writeSecureHWGFile.setOnClickListener(this);
        AppInfoText.setText("Application Version: " + appVersion + "  |  " +"SDK Version: " + HexConstants.VERSION_NAME );
        checkKeystrokesForUI();

        // Register broadcast receiver that catches USB permission intent
        mUsbManager = (UsbManager) getSystemService(Context.USB_SERVICE);
        HashMap<String, UsbDevice> deviceList = mUsbManager.getDeviceList();
        Iterator<UsbDevice> deviceIterator = deviceList.values().iterator();
        while(deviceIterator.hasNext()) {
            UsbDevice device = deviceIterator.next();
            if (device.getVendorId() == HexConstants.VENDOR_ID) {
                if (device.getProductId() == HexConstants.PLUS_PRODUCT_ID | device.getProductId() == HexConstants.BLE_PRODUCT_ID) {
                    mPermissionIntent = PendingIntent.getBroadcast(this, 0, new Intent(ACTION_USB_PERMISSION), 0);
                    if (!mUsbManager.hasPermission(device)) {
                        mUsbManager.requestPermission(device, mPermissionIntent);
                    } else {
                        hasPermissionToCommunicate = true;
                    }
                }
            }
        }
    }

    protected void onPause() {
        unregisterReceiver(mUsbReceiver);
        Log.d(TAG, "Pausing Activity");
        super.onPause();
    }

    protected void onResume() {
        super.onResume();
        Log.d(TAG, "Resuming Activity");
        registerReceiver(mUsbReceiver, filter);
    }
    @Override
    protected void onDestroy() {
        super.onDestroy();
        Log.w(TAG, "Destroying Activity");
    }
    @Override
    public void onClick(View view) {
        try
        {
            byte beepNumber = 2; //for number of beeps to be passed in BeepNow()
            switch (view.getId())
            {
                case R.id.connectButton:
                    connectTheDevice();
                    break;

                case R.id.beepnowButton:
                    reader[readerNo].BeepNow(beepNumber,Boolean.TRUE);
                    break;

                case R.id.getActiveIdButton:
                    sdkOutput.setText(null);
                    String buffer = getActiveIdOutput();
                    sdkOutput.setText(buffer);
                    break;

                case R.id.disconnectButton:
                    disconnectTheDevice();
                    break;

                case R.id.readHWGFileButton:
                    readHWGFile();
                    break;

                case R.id.writeHWGFileButton:
                    writeHWGFile();
                    break;
                
                case R.id.readSecureHWGFileButton:
                    readSecureHWGFile();
                    break;

                case R.id.writeSecureHWGFileButton:
                    writeSecureHWGFile();
                    break;
                    
                default:
                    break;
            }
        }
        catch (SDKException ive) {
            Log.e(TAG, "Invalid value of input or Exception in USB Communication", ive);
        }
        catch(Exception exception) {
            Log.e(TAG, "Exception occured", exception);
        }

    }

    @Override
    public void onConfigurationChanged(Configuration newConfig)
    {
        super.onConfigurationChanged(newConfig);
        Log.d(TAG, "Configuration Changed - Keyboard: " + newConfig.keyboard);
    }
    //This functions connects to the reader and also populates the UI of the application.
    public void connectTheDevice() throws SDKException, CloneNotSupportedException {
        reader = Enumerate.USBConnect(getApplicationContext());
        //Calling the static method of the Singleton class Enumerate returns an array of connected readers.

        if (reader == null) {
            Toast.makeText(this, "No Device Connected", Toast.LENGTH_LONG).show();
            return;
        }
        //calling the getPartNumberString API, it returns a String value
        String partNumberString = reader[readerNo].getPartNumberString();
        readerPartNumber.setText(partNumberString);

        //display the maximum configuration possessed by a reader
        Integer maximumConfiguration = reader[readerNo].GetMaxConfig() + 1;
        maxConfigurationNumber.setText(maximumConfiguration + "");

        //A call to readCfg reads all the values  of the specified configuration and stores them in local memory.
        // Developer must call this API before attempting to access/alter any settings.
        reader[readerNo].ReadCfg(configurationIndex);

        //to have disableKeystrokes checkbox checked in UI if internally haltKBSend is enabled i.e taking the state previously saved
        final ConfigurationFlags configurationFlags = reader[readerNo].GetFlags(configurationIndex);
        if (configurationFlags.getbHaltKBSnd())
            disableKeystrokes.setChecked(true);
        else
            disableKeystrokes.setChecked(false);

    }

    public void disconnectTheDevice() {
        //API for disconnecting the readers.
        Enumerate.USBDisconnect();
        readerPartNumber.setText(null);
        maxConfigurationNumber.setText(null);
        sdkOutput.setText(null);
    }

    public void checkKeystrokesForUI(){
        //when disableKeystrokes checkbox is checked or unchecked is done in UI then the change is saved in the reader
        disableKeystrokes.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {

            @Override
            public void onCheckedChanged(CompoundButton compoundButton, boolean isChecked) {

                //Checking if reader is not null so that when no reader is connected, it should not try to change any of the Configuration Flags in savingTheConfiguration method
                if (reader != null)
                {
                    try {
                        savingTheConfiguration();
                    }
                    catch (SDKException ive) {
                        Log.e(TAG, "Invalid value of input or Exception in USB Communication", ive);
                    }
                    catch(Exception exception) {
                        Log.e(TAG, "Exception ", exception);
                    }
                }
            }
        });
    }


    public void savingTheConfiguration() throws SDKException, InterruptedException, CloneNotSupportedException {
        //Pulling the ConfigurationFlags object and changing the value of haltKBSnd in the object and writing the changed value using WriteCfg
        ConfigurationFlags configurationFlags = reader[readerNo].GetFlags(configurationIndex);
        configurationFlags.setbHaltKBSnd(disableKeystrokes.isChecked());
        reader[readerNo].SetFlags(configurationFlags,configurationIndex);
        reader[readerNo].WriteCfg(configurationIndex);
    }


    public String getActiveIdOutput()throws SDKException {
        //Added sleep as mentioned in the API documentation
        try{
            Thread.sleep(250);
        }
        catch(InterruptedException ie){
            Log.e(TAG, "sleep interrupted", ie);
        }
        short bits;
        short size = 32;
        int[] buf = new int[size];
        //Calling the API for GetActiveID
        bits = reader[readerNo].GetActiveID(buf, size);
        if (bits == 0) {
            Toast.makeText(this, "No id found, Please put card on the reader", Toast.LENGTH_LONG).show();
        } else {
            String buffer = "";
            String temp = "";
            int bytesToRead = (bits + 7) / 8;
            if (bytesToRead < 8) {
                bytesToRead = 8;
            }
            for (int i = 0; i < bytesToRead; i++) {
                temp = Integer.toHexString(buf[i]);
                buffer = buffer.concat(temp);
                buffer = buffer.concat(" ");
            }
            Toast.makeText(this, bits + " Bits: " + buffer.toUpperCase(), Toast.LENGTH_LONG).show();
            return buffer;
        }
        return null;
    }

    //This function call makes sure that the application has required permission to write to storage.
    public void writeHWGFile() {
        if (this.checkSelfPermission(WRITE_EXTERNAL_STORAGE) != PackageManager.PERMISSION_GRANTED) {
            String[] permissions = {WRITE_EXTERNAL_STORAGE};
            requestPermissions(permissions, 1);
        } else {
            writeHWG();
        }
    }

    //This function call makes sure that the application has required permission to write to storage.
    public void writeSecureHWGFile() {
        if (this.checkSelfPermission(WRITE_EXTERNAL_STORAGE) != PackageManager.PERMISSION_GRANTED) {
            String[] permissions = {WRITE_EXTERNAL_STORAGE};
            requestPermissions(permissions, 1);
        } else {
            writeSecureHWG();
        }
    }

    //This function call makes sure that the application has required permission to read from storage.
    public void readHWGFile() {
        if (this.checkSelfPermission(WRITE_EXTERNAL_STORAGE) != PackageManager.PERMISSION_GRANTED) {
            String[] permissions = {WRITE_EXTERNAL_STORAGE};
            requestPermissions(permissions, 1);
        } else {
            readHWG();
        }
    }

    //This function call makes sure that the application has required permission to read from storage.
    public void readSecureHWGFile() {
        if (this.checkSelfPermission(WRITE_EXTERNAL_STORAGE) != PackageManager.PERMISSION_GRANTED) {
            String[] permissions = {WRITE_EXTERNAL_STORAGE};
            requestPermissions(permissions, 1);
        } else {
            readSecureHWG();
        }
    }

    //The function gets called when the user replies to the askPermission dialog box. The application
    //proceeds for Read/Write HWG file based on the button pressed.
    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (requestCode == WRITE_REQUEST_CODE && resultCode == Activity.RESULT_OK) {
            Uri uri = data.getData();
            try {
                OutputStream outputStream = this.getContentResolver().openOutputStream(uri);
                reader[0].WriteDevCfgToFile(outputStream);
                Toast.makeText(this, "HWG File Written", Toast.LENGTH_SHORT).show();
            } catch (Exception e) {
                Toast.makeText(this, e.getMessage(), Toast.LENGTH_SHORT).show();
            }
        }
        if (requestCode == SECURE_WRITE_REQUEST_CODE && resultCode == Activity.RESULT_OK) {
            Uri uri = data.getData();
            try {
                OutputStream outputStream = this.getContentResolver().openOutputStream(uri);
                FileInputStream inputStream = (FileInputStream)this.getContentResolver().openInputStream(uri);
                reader[0].WriteDevCfgToSecureFile(outputStream, inputStream);
                Toast.makeText(this, "Secure HWG File Written", Toast.LENGTH_SHORT).show();
            } catch (Exception e) {
                Toast.makeText(this, e.getMessage(), Toast.LENGTH_SHORT).show();
            }
        }
        if (requestCode == READ_REQUEST_CODE && resultCode == Activity.RESULT_OK) {
            Uri uri = data.getData();
            try {
                FileInputStream inputStream = (FileInputStream)this.getContentResolver().openInputStream(uri);
                reader[0].ReadDevCfgFmFile(inputStream);
                Toast.makeText(this, "HWG File Read", Toast.LENGTH_SHORT).show();
            }
            catch (Exception e) {
                Toast.makeText(this, e.getMessage(), Toast.LENGTH_SHORT).show();
            }
        }
        if (requestCode == SECURE_READ_REQUEST_CODE && resultCode == Activity.RESULT_OK) {
            Uri uri = data.getData();
            try {
                FileInputStream inputStream = (FileInputStream)this.getContentResolver().openInputStream(uri);
                reader[0].ReadDevCfgFmSecureFile(inputStream);
                Toast.makeText(this, "Secure HWG File Read", Toast.LENGTH_SHORT).show();
            }
            catch (Exception e) {
                Toast.makeText(this, e.getMessage(), Toast.LENGTH_SHORT).show();
            }
        }
    }

    //Asks user for file name and location and based on the selection, the SDK API is called
    public void writeHWG() {
        try {
            Intent intent = new Intent(Intent.ACTION_CREATE_DOCUMENT);
            intent.addCategory(Intent.CATEGORY_OPENABLE);
            intent.setType("*/*");
            String partNumber = reader[0].getPartNumberString();
            intent.putExtra(Intent.EXTRA_TITLE, partNumber+ ".hwg+");
            startActivityForResult(intent, WRITE_REQUEST_CODE);
        } catch (Exception e) {
            Toast.makeText(this, e.getMessage(), Toast.LENGTH_SHORT).show();
        }
    }

    //Asks user for file name and location and based on the selection, the SDK API is called
    public void writeSecureHWG() {
        try {
            Intent intent = new Intent(Intent.ACTION_CREATE_DOCUMENT);
            intent.addCategory(Intent.CATEGORY_OPENABLE);
            intent.setType("*/*");
            String partNumber = reader[0].getPartNumberString();
            intent.putExtra(Intent.EXTRA_TITLE, partNumber+"Secure" + ".hwg+");
            startActivityForResult(intent, SECURE_WRITE_REQUEST_CODE);
        } catch (Exception e) {
            Toast.makeText(this, e.getMessage(), Toast.LENGTH_SHORT).show();
        }
    }

    //Asks user to select the HWG file and based on the selection, the SDK API is called
    public void readHWG() {
        try {
            Intent intent = new Intent(Intent.ACTION_OPEN_DOCUMENT);
            intent.addCategory(Intent.CATEGORY_OPENABLE);
            intent.setType("*/*");
            startActivityForResult(intent, READ_REQUEST_CODE);
        } catch (Exception e) {
            Toast.makeText(this, e.getMessage(), Toast.LENGTH_SHORT).show();
        }
    }

    //Asks user to select the HWG file and based on the selection, the SDK API is called
    public void readSecureHWG() {
        try {
            Intent intent = new Intent(Intent.ACTION_OPEN_DOCUMENT);
            intent.addCategory(Intent.CATEGORY_OPENABLE);
            intent.setType("*/*");
            startActivityForResult(intent, SECURE_READ_REQUEST_CODE);
        } catch (Exception e) {
            Toast.makeText(this, e.getMessage(), Toast.LENGTH_SHORT).show();
        }
    }
}